"""Copyright 2016 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License."""

from traceutils.record.record import Record


class InstructionRecord(Record):
    """Instruction trace record."""

    CPU = "cpu"
    COND_CODE_PASS = "cond_code_pass"
    INST_ID = "inst_id"
    ADDR = "addr"
    OPCODE = "opcode"
    INST_SET = "ins_set"
    MODE = "mode"
    SEC = "sec"
    DISASM = "disasm"

    def __init__(self, time, scale, cpu, cond_code_pass, inst_id, addr, opcode, inst_set, mode, sec, disasm):
        """Default constructor."""
        Record.__init__(self, time, scale)
        self[self.CPU] = cpu
        self[self.COND_CODE_PASS] = cond_code_pass
        self[self.INST_ID] = inst_id
        self[self.ADDR] = addr
        self[self.OPCODE] = opcode
        self[self.INST_SET] = inst_set
        self[self.MODE] = mode
        self[self.SEC] = sec
        self[self.DISASM] = disasm

    def get_cpu(self):
        """Returns the processor that gave the instruction."""
        return self[self.CPU]

    def get_cond_code_pass(self):
        """Returns whether the instruction passed the condition code."""
        return self[self.COND_CODE_PASS]

    def get_inst_id(self):
        """Returns the tick count of this processor."""
        return self[self.INST_ID]

    def get_addr(self):
        """Returns the source address for this instruction, in hexadecimal format (virtual address)."""
        return self[self.ADDR]

    def get_opcode(self):
        """Returns the 16-bit/32-bit hexadecimal opcode of the instruction."""
        return self[self.OPCODE]

    def get_inst_set(self):
        """Returns the instruction set identifier."""
        return self[self.INST_SET]

    def get_mode(self):
        """Returns the processor execution mode (svc, irq, fiq, usr, mon, sys, abt, und)."""
        return self[self.MODE]

    def get_sec(self):
        """Returns the processor security state (s or ns)."""
        return self[self.SEC]

    def get_disasm(self):
        """Returns the disassembly of the instruction."""
        return self[self.DISASM]
